@@ -15,10 +15,11 @@ from scipy import linalg
 from .tree import dir_tree_find
 from .tag import find_tag
 from .constants import FIFF
-from .pick import pick_types
+from .pick import pick_types, pick_info
 from .write import (write_int, write_float, write_string, write_name_list,
                     write_float_matrix, end_block, start_block)
-from ..utils import logger, verbose, warn, fill_doc
+from ..defaults import _BORDER_DEFAULT, _EXTRAPOLATE_DEFAULT
+from ..utils import logger, verbose, warn, fill_doc, _validate_type
 
 
 class Projection(dict):
@@ -30,8 +31,8 @@ class Projection(dict):
     def __repr__(self):  # noqa: D105
         s = "%s" % self['desc']
         s += ", active : %s" % self['active']
-        s += ", n_channels : %s" % self['data']['ncol']
-        return "<Projection  |  %s>" % s
+        s += f", n_channels : {len(self['data']['col_names'])}"
+        return "<Projection | %s>" % s
 
     # speed up info copy by taking advantage of mutability
     def __deepcopy__(self, memodict):
@@ -51,16 +52,14 @@ class Projection(dict):
     def plot_topomap(self, info, cmap=None, sensors=True,
                      colorbar=False, res=64, size=1, show=True,
                      outlines='head', contours=6, image_interp='bilinear',
-                     axes=None, vlim=(None, None), layout=None,
-                     sphere=None, border=0):
+                     axes=None, vlim=(None, None), sphere=None,
+                     border=_BORDER_DEFAULT):
         """Plot topographic maps of SSP projections.
 
         Parameters
         ----------
         info : instance of Info | None
-            The measurement information to use to determine the layout. If both
-            ``info`` and ``layout`` are provided, the layout will take
-            precedence.
+            The measurement information to use to determine the layout.
         %(proj_topomap_kwargs)s
         %(topomap_sphere_auto)s
         %(topomap_border)s
@@ -77,7 +76,7 @@ class Projection(dict):
         from ..viz.topomap import plot_projs_topomap
         return plot_projs_topomap(self, info, cmap, sensors, colorbar,
                                   res, size, show, outlines,
-                                  contours, image_interp, axes, vlim, layout,
+                                  contours, image_interp, axes, vlim,
                                   sphere=sphere, border=border)
 
 
@@ -153,9 +152,14 @@ class ProjMixin(object):
                                              check_active=False, sort=False)
         return self
 
-    def apply_proj(self):
+    @verbose
+    def apply_proj(self, verbose=None):
         """Apply the signal space projection (SSP) operators to the data.
 
+        Parameters
+        ----------
+        %(verbose_meth)s
+
         Returns
         -------
         self : instance of Raw | Epochs | Evoked
@@ -239,21 +243,30 @@ class ProjMixin(object):
         if isinstance(idx, str) and idx == 'all':
             idx = list(range(len(self.info['projs'])))
         idx = np.atleast_1d(np.array(idx, int)).ravel()
-        if any(self.info['projs'][ii]['active'] for ii in idx):
-            raise ValueError('Cannot remove projectors that have already '
-                             'been applied')
+
+        for ii in idx:
+            proj = self.info['projs'][ii]
+            if (proj['active'] and
+                    set(self.info['ch_names']) &
+                    set(proj['data']['col_names'])):
+                msg = (f'Cannot remove projector that has already been '
+                       f'applied, unless you first remove all channels it '
+                       f'applies to. The problematic projector is: {proj}')
+                raise ValueError(msg)
+
         keep = np.ones(len(self.info['projs']))
         keep[idx] = False  # works with negative indexing and does checks
         self.info['projs'] = [p for p, k in zip(self.info['projs'], keep) if k]
         return self
 
     @fill_doc
-    def plot_projs_topomap(self, ch_type=None, layout=None, cmap=None,
+    def plot_projs_topomap(self, ch_type=None, cmap=None,
                            sensors=True, colorbar=False, res=64, size=1,
                            show=True, outlines='head', contours=6,
                            image_interp='bilinear', axes=None,
-                           vlim=(None, None), sphere=None, extrapolate='box',
-                           border=0):
+                           vlim=(None, None), sphere=None,
+                           extrapolate=_EXTRAPOLATE_DEFAULT,
+                           border=_BORDER_DEFAULT):
         """Plot SSP vector.
 
         Parameters
@@ -263,8 +276,6 @@ class ProjMixin(object):
             ted in pairs and the RMS for each pair is plotted. If None
             (default), it will return all channel types present. If a list of
             ch_types is provided, it will return multiple figures.
-        layout : object
-            Deprecated, do not use.
         %(proj_topomap_kwargs)s
         %(topomap_sphere_auto)s
         %(topomap_extrapolate)s
@@ -290,6 +301,29 @@ class ProjMixin(object):
             raise ValueError("Info is missing projs. Nothing to plot.")
         return fig
 
+    def _reconstruct_proj(self, mode='accurate', origin='auto'):
+        from ..forward import _map_meg_or_eeg_channels
+        if len(self.info['projs']) == 0:
+            return self
+        self.apply_proj()
+        for kind in ('meg', 'eeg'):
+            kwargs = dict(meg=False)
+            kwargs[kind] = True
+            picks = pick_types(self.info, **kwargs)
+            if len(picks) == 0:
+                continue
+            info_from = pick_info(self.info, picks)
+            info_to = info_from.copy()
+            info_to['projs'] = []
+            if kind == 'eeg' and _has_eeg_average_ref_proj(info_from['projs']):
+                info_to['projs'] = [
+                    make_eeg_average_ref_proj(info_to, verbose=False)]
+            mapping = _map_meg_or_eeg_channels(
+                info_from, info_to, mode=mode, origin=origin)
+            self.data[..., picks, :] = np.matmul(
+                mapping, self.data[..., picks, :])
+        return self
+
 
 def _proj_equal(a, b, check_active=True):
     """Test if two projectors are equal."""
@@ -328,18 +362,23 @@ def _read_proj(fid, node, verbose=None):
     if len(nodes) == 0:
         return projs
 
-    tag = find_tag(fid, nodes[0], FIFF.FIFF_NCHAN)
-    if tag is not None:
-        global_nchan = int(tag.data)
+    # This might exist but we won't use it:
+    # global_nchan = None
+    # tag = find_tag(fid, nodes[0], FIFF.FIFF_NCHAN)
+    # if tag is not None:
+    #     global_nchan = int(tag.data)
 
     items = dir_tree_find(nodes[0], FIFF.FIFFB_PROJ_ITEM)
     for item in items:
         #   Find all desired tags in one item
-        tag = find_tag(fid, item, FIFF.FIFF_NCHAN)
-        if tag is not None:
-            nchan = int(tag.data)
-        else:
-            nchan = global_nchan
+
+        # This probably also exists but used to be written incorrectly
+        # sometimes
+        # tag = find_tag(fid, item, FIFF.FIFF_NCHAN)
+        # if tag is not None:
+        #     nchan = int(tag.data)
+        # else:
+        #     nchan = global_nchan
 
         tag = find_tag(fid, item, FIFF.FIFF_DESCRIPTION)
         if tag is not None:
@@ -351,13 +390,6 @@ def _read_proj(fid, node, verbose=None):
             else:
                 raise ValueError('Projection item description missing')
 
-        # XXX : is this useful ?
-        # tag = find_tag(fid, item, FIFF.FIFF_PROJ_ITEM_CH_NAME_LIST)
-        # if tag is not None:
-        #     namelist = tag.data
-        # else:
-        #     raise ValueError('Projection item channel list missing')
-
         tag = find_tag(fid, item, FIFF.FIFF_PROJ_ITEM_KIND)
         if tag is not None:
             kind = int(tag.data)
@@ -402,6 +434,9 @@ def _read_proj(fid, node, verbose=None):
             raise ValueError('Number of channel names does not match the '
                              'size of data matrix')
 
+        # just always use this, we used to have bugs with writing the
+        # number correctly...
+        nchan = len(names)
         #   Use exactly the same fields in data as in a named matrix
         one = Projection(kind=kind, active=active, desc=desc,
                          data=dict(nrow=nvec, ncol=nchan, row_names=None,
@@ -412,14 +447,11 @@ def _read_proj(fid, node, verbose=None):
 
     if len(projs) > 0:
         logger.info('    Read a total of %d projection items:' % len(projs))
-        for k in range(len(projs)):
-            if projs[k]['active']:
-                misc = 'active'
-            else:
-                misc = ' idle'
-            logger.info('        %s (%d x %d) %s'
-                        % (projs[k]['desc'], projs[k]['data']['nrow'],
-                           projs[k]['data']['ncol'], misc))
+        for proj in projs:
+            misc = 'active' if proj['active'] else ' idle'
+            logger.info(f'        {proj["desc"]} '
+                        f'({proj["data"]["nrow"]} x '
+                        f'{len(proj["data"]["col_names"])}) {misc}')
 
     return projs
 
@@ -439,11 +471,17 @@ def _write_proj(fid, projs):
     """
     if len(projs) == 0:
         return
+
+    # validation
+    _validate_type(projs, (list, tuple), 'projs')
+    for pi, proj in enumerate(projs):
+        _validate_type(proj, Projection, f'projs[{pi}]')
+
     start_block(fid, FIFF.FIFFB_PROJ)
 
     for proj in projs:
         start_block(fid, FIFF.FIFFB_PROJ_ITEM)
-        write_int(fid, FIFF.FIFF_NCHAN, proj['data']['ncol'])
+        write_int(fid, FIFF.FIFF_NCHAN, len(proj['data']['col_names']))
         write_name_list(fid, FIFF.FIFF_PROJ_ITEM_CH_NAME_LIST,
                         proj['data']['col_names'])
         write_string(fid, FIFF.FIFF_NAME, proj['desc'])
@@ -589,6 +627,7 @@ def _make_projector(projs, ch_names, bads=(), include_active=True,
                 p['data']['data'] = this_vecs[sel].T
                 p['data']['col_names'] = [p['data']['col_names'][ii]
                                           for ii in vecsel]
+                p['data']['ncol'] = len(p['data']['col_names'])
             nvec += p['data']['nrow']
 
     #   Check whether all of the vectors are exactly zero
