@@ -10,6 +10,8 @@
 # License: Simplified BSD
 
 from ._brain import _Brain
-from ._timeviewer import _TimeViewer, _LinkViewer
+from ._scraper import _BrainScraper
+from ._timeviewer import _TimeViewer
+from ._linkviewer import _LinkViewer
 
 __all__ = ['_Brain']
